﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using Infragistics.Windows;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;


namespace MatrixGridProblem
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        public ObservableCollection<Trade> Trades { get; set; }

        public MainWindow()
        {
            DataContext = this;
            InitializeComponent();

            Trades = new ObservableCollection<Trade>();
            Trades.Add(new Trade { Symbol = "IBM", Broker = "DEU", Amount = 100 });
            Trades.Add(new Trade { Symbol = "MSFT", Broker = "DEU", Amount = -150 });
            Trades.Add(new Trade { Symbol = "MSFT", Broker = "BOA", Amount = 300 });

            IEnumerable<TradeGroup> TradeGroups = Trades
                .GroupBy(x => x.Symbol)
                .Select(x => new TradeGroup 
                {
                    Symbol = x.Key,
                    TotalAmount = x.Sum(y => y.Amount),
                    Trades = x.ToDictionary<Trade,string>(t => t.Broker) // assume brokers are unique for this example
                });

            grid1.DataSource = TradeGroups;
        }


        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(name));
            }
        }
        #endregion

        private void XamDataGrid_FieldLayoutInitialized(object sender, Infragistics.Windows.DataPresenter.Events.FieldLayoutInitializedEventArgs e)
        {
            // Add columns to the grid - one column for each distinct broker

            foreach (string broker in Trades.Select(x => x.Broker).OrderBy(x => x).Distinct())
            {
                // This binding provides content for the field
                MultiBinding contentBinding = new MultiBinding { Converter = new BrokerColumnConverter(), ConverterParameter = 0 };
                contentBinding.Bindings.Add(new Binding());                             // The row being bound
                contentBinding.Bindings.Add(new Binding { Source = broker });           // The broker that identifies the column. 

                // This binding is used by the cellvaluepresenter for the background color
                MultiBinding backgroundBinding = new MultiBinding { Converter = new BrokerColumnConverter(), ConverterParameter = 1 };
                backgroundBinding.Bindings.Add(new Binding());                             // The row being bound
                contentBinding.Bindings.Add(new Binding { Source = broker });           // The broker that identifies the column. 

                // This binding is used by the cellvaluepresenter for the foreground color
                MultiBinding foregroundBinding = new MultiBinding { Converter = new BrokerColumnConverter(), ConverterParameter = 2 };
                foregroundBinding.Bindings.Add(new Binding());                             // The row being bound
                contentBinding.Bindings.Add(new Binding { Source = broker });           // The broker that identifies the column. 

                // Style for cell background / foreground
                Style cellValuePresenterStyle = new Style { TargetType = typeof(Infragistics.Windows.DataPresenter.CellValuePresenter) };
                cellValuePresenterStyle.Setters.Add(new Setter { Property = BackgroundProperty, Value = backgroundBinding });
                cellValuePresenterStyle.Setters.Add(new Setter { Property = ForegroundProperty, Value = foregroundBinding });

                // Style for column header
                Style headerBackgroundStyle = new Style { TargetType = typeof(LabelPresenter) };
                headerBackgroundStyle.Setters.Add(new Setter { Property = BackgroundProperty, Value = new SolidColorBrush(Colors.Orange) });
                headerBackgroundStyle.Setters.Add(new Setter { Property = HeightProperty, Value = Convert.ToDouble(33) });

                // Can we use a Field here, versus UnboundField?
                UnboundField unboundField = new UnboundField
                {
                    Name = Guid.NewGuid().ToString(),      // Generate a bogus field name so the summary calculator knows what to total
                    Label = broker,
                    DataType = typeof(string),
                    Binding = contentBinding,
                    Settings = new FieldSettings
                    {
                        LabelPresenterStyle = headerBackgroundStyle,
                        EditorType = typeof(Infragistics.Windows.Editors.XamTextEditor),
                        CellValuePresenterStyle = cellValuePresenterStyle
                    }
                };

                grid1.FieldLayouts[0].Fields.Add(unboundField);
            }
        }


        public class BrokerColumnConverter : IMultiValueConverter
        {
            public object Convert(object[] value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
            {
                int action = System.Convert.ToInt16(parameter);
                
                if (action != 0)  // remove this line to get background / foreground color (app will crash)
                    return null;  // remove this line to get background / foreground color (app will crash)

                // Problem occurs here when trying to cast to TradeGroup. The DataItem is null
                // Note that the binding for the unboundfield works just fine - the problem is with the bindings for the cellvaluepresenter
                TradeGroup boundItem = (TradeGroup)value[0];  
                string broker = System.Convert.ToString(value[1]);  // broker that identifies the column
                Trade brokerResult = null;
                decimal result = 0;

                if (boundItem.Trades.TryGetValue(broker, out brokerResult))
                    result = brokerResult.Amount;

                if (action == 0)                                                            // return cell content
                    return result;
                else if (action == 1)                                                       // return background color
                    return new SolidColorBrush(result > 0 ? Colors.Green : Colors.Red);
                else                                                                        // return foreground color
                    return new SolidColorBrush(Colors.White);
            }

            public object[] ConvertBack(object value, Type[] targetType, object parameter, System.Globalization.CultureInfo culture)
            {
                throw new NotImplementedException();
            }
        }
    }


    public class Trade
    {
        public string Symbol { get; set; }
        public string Broker { get; set; }
        public decimal Amount { get; set; }
    }

    public class TradeGroup  // Trades grouped by Symbol
    {
        public string Symbol { get; set; }
        public decimal TotalAmount { get; set; }
        public Dictionary<string, Trade> Trades { get; set; }

        public TradeGroup()
        {
            Trades = new Dictionary<string, Trade>();
        }
    }
}
